@@ -3,6 +3,7 @@
 #include "vw/core/setup_base.h"
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
 #include <string>
 
 namespace VW
@@ -10,7 +11,7 @@ namespace VW
 class cached_learner : public setup_base_i
 {
 public:
-  VW::LEARNER::base_learner* setup_base_learner() override { return _cached; }
+  std::shared_ptr<VW::LEARNER::learner> setup_base_learner() override { return _cached; }
 
   operator bool() const { return !(_cached == nullptr); }
 
@@ -20,10 +21,12 @@ public:
     _all_ptr = &all;
   }
 
-  cached_learner(VW::LEARNER::base_learner* learner = nullptr) : _cached(learner) {}
+  cached_learner() : _cached(nullptr) {}
 
-  cached_learner(VW::workspace& all, VW::config::options_i& options, VW::LEARNER::base_learner* learner = nullptr)
-      : _cached(learner)
+  cached_learner(std::shared_ptr<VW::LEARNER::learner> learner) : _cached(std::move(learner)) {}
+
+  cached_learner(VW::workspace& all, VW::config::options_i& options, std::shared_ptr<VW::LEARNER::learner> learner)
+      : _cached(std::move(learner))
   {
     delayed_state_attach(all, options);
   }
@@ -35,7 +38,7 @@ public:
   std::string get_setupfn_name(reduction_setup_fn) override { return ""; }
 
 private:
-  VW::LEARNER::base_learner* _cached = nullptr;
+  std::shared_ptr<VW::LEARNER::learner> _cached;
   VW::config::options_i* _options_impl = nullptr;
   VW::workspace* _all_ptr = nullptr;
 };
