@@ -10,8 +10,9 @@
 
 namespace VW
 {
+// This must be a raw function pointer for use in C# bindings
 using trace_message_t = void (*)(void*, const std::string&);
-}
+}  // namespace VW
 
 using trace_message_t VW_DEPRECATED(
     "Moved in VW namespace. Global symbol will be removed in VW 10.") = VW::trace_message_t;
\ No newline at end of file
