@@ -7,6 +7,8 @@
 #include "vw/core/version.h"
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
+
 namespace VW
 {
 namespace reductions
@@ -24,6 +26,6 @@ public:
   VW::version_struct _model_version;
 };
 
-VW::LEARNER::base_learner* active_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> active_setup(VW::setup_base_i& stack_builder);
 }  // namespace reductions
 }  // namespace VW
