@@ -6,10 +6,12 @@
 
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
+
 namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* active_cover_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> active_cover_setup(VW::setup_base_i& stack_builder);
 }
 }  // namespace VW
