@@ -2,10 +2,12 @@
 
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
+
 namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* cb_dro_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> cb_dro_setup(VW::setup_base_i& stack_builder);
 }
 }  // namespace VW
\ No newline at end of file
