@@ -5,11 +5,12 @@
 
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
 #include <vector>
 namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* cb_explore_adf_greedy_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> cb_explore_adf_greedy_setup(VW::setup_base_i& stack_builder);
 }  // namespace reductions
 }  // namespace VW
