@@ -6,10 +6,12 @@
 
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
+
 namespace VW
 {
 namespace reductions
 {
-LEARNER::base_learner* cb_explore_adf_rnd_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> cb_explore_adf_rnd_setup(VW::setup_base_i& stack_builder);
 }  // namespace reductions
 }  // namespace VW
