@@ -5,10 +5,12 @@
 #pragma once
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
+
 namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* classweight_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> classweight_setup(VW::setup_base_i& stack_builder);
 }  // namespace reductions
 }  // namespace VW
