@@ -6,6 +6,7 @@
 
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
 #include <set>
 #include <vector>
 
@@ -13,6 +14,6 @@ namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* generate_interactions_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> generate_interactions_setup(VW::setup_base_i& stack_builder);
 }
 }  // namespace VW
