@@ -4,10 +4,12 @@
 
 #pragma once
 #include "vw/core/vw_fwd.h"
+
+#include <memory>
 namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* lrq_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> lrq_setup(VW::setup_base_i& stack_builder);
 }
 }  // namespace VW
\ No newline at end of file
