@@ -5,12 +5,13 @@
 
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
+
 namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* metrics_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> metrics_setup(VW::setup_base_i& stack_builder);
 void output_metrics(VW::workspace& all);
-void additional_metrics(VW::workspace& all, VW::metric_sink& sink);
 }  // namespace reductions
 }  // namespace VW
