@@ -6,11 +6,13 @@
 #include "vw/core/v_array.h"
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
+
 namespace VW
 {
 namespace reductions
 {
-VW::LEARNER::base_learner* mwt_setup(VW::setup_base_i& stack_builder);
+std::shared_ptr<VW::LEARNER::learner> mwt_setup(VW::setup_base_i& stack_builder);
 }
 }  // namespace VW
 
