@@ -6,20 +6,23 @@
 
 #include "vw/core/vw_fwd.h"
 
+#include <memory>
 #include <string>
 
 namespace VW
 {
 class setup_base_i;
 
-using reduction_setup_fn = VW::LEARNER::base_learner* (*)(VW::setup_base_i&);
+// use raw function pointer here instead of std::function because
+// this type should be hashable to be used in std::unordered_map
+using reduction_setup_fn = std::shared_ptr<VW::LEARNER::learner> (*)(VW::setup_base_i&);
 
 class setup_base_i
 {
 public:
   virtual void delayed_state_attach(VW::workspace&, VW::config::options_i&) = 0;
 
-  virtual VW::LEARNER::base_learner* setup_base_learner() = 0;
+  virtual std::shared_ptr<VW::LEARNER::learner> setup_base_learner() = 0;
 
   // this one we can share freely
   virtual VW::config::options_i* get_options() = 0;
