@@ -67,7 +67,7 @@ public:
   bool use_domination = false;
 
   VW::workspace* all = nullptr;  // statistics, loss
-  VW::LEARNER::base_learner* l = nullptr;
+  VW::LEARNER::learner* l = nullptr;
 
   VW::v_array<lq_data> query_data;
 
@@ -100,8 +100,8 @@ float binary_search(float fhat, float delta, float sens, float tol)
 }
 
 template <bool is_learn, bool is_simulation>
-inline void inner_loop(cs_active& cs_a, single_learner& base, VW::example& ec, uint32_t i, float cost,
-    uint32_t& prediction, float& score, float& partial_prediction, bool query_this_label, bool& query_needed)
+inline void inner_loop(cs_active& cs_a, learner& base, VW::example& ec, uint32_t i, float cost, uint32_t& prediction,
+    float& score, float& partial_prediction, bool query_this_label, bool& query_needed)
 {
   base.predict(ec, i - 1);
   if (is_learn)
@@ -152,7 +152,7 @@ inline void inner_loop(cs_active& cs_a, single_learner& base, VW::example& ec, u
   VW_ADD_PASSTHROUGH_FEATURE(ec, i, ec.partial_prediction);
 }
 
-inline void find_cost_range(cs_active& cs_a, single_learner& base, VW::example& ec, uint32_t i, float delta, float eta,
+inline void find_cost_range(cs_active& cs_a, learner& base, VW::example& ec, uint32_t i, float delta, float eta,
     float& min_pred, float& max_pred, bool& is_range_large)
 {
   float tol = 1e-6f;
@@ -188,7 +188,7 @@ inline void find_cost_range(cs_active& cs_a, single_learner& base, VW::example&
 }
 
 template <bool is_learn, bool is_simulation>
-void predict_or_learn(cs_active& cs_a, single_learner& base, VW::example& ec)
+void predict_or_learn(cs_active& cs_a, learner& base, VW::example& ec)
 {
   VW::cs_label ld = ec.l.cs;
 
@@ -376,7 +376,7 @@ void print_update_cs_active(VW::workspace& all, VW::shared_data& /* sd */, const
 }
 }  // namespace
 
-base_learner* VW::reductions::cs_active_setup(VW::setup_base_i& stack_builder)
+std::shared_ptr<VW::LEARNER::learner> VW::reductions::cs_active_setup(VW::setup_base_i& stack_builder)
 {
   options_i& options = *stack_builder.get_options();
   VW::workspace& all = *stack_builder.get_all_pointer();
@@ -434,12 +434,16 @@ base_learner* VW::reductions::cs_active_setup(VW::setup_base_i& stack_builder)
 
   if (!options.was_supplied("adax")) { all.logger.err_warn("--cs_active should be used with --adax"); }
 
-  all.set_minmax(all.sd, data->cost_max);
-  all.set_minmax(all.sd, data->cost_min);
+  if (all.set_minmax)
+  {
+    all.set_minmax(data->cost_max);
+    all.set_minmax(data->cost_min);
+  }
+
   for (uint32_t i = 0; i < data->num_classes + 1; i++) { data->examples_by_queries.push_back(0); }
 
-  void (*learn_ptr)(cs_active & cs_a, single_learner & base, VW::example & ec);
-  void (*predict_ptr)(cs_active & cs_a, single_learner & base, VW::example & ec);
+  void (*learn_ptr)(cs_active & cs_a, learner & base, VW::example & ec);
+  void (*predict_ptr)(cs_active & cs_a, learner & base, VW::example & ec);
   std::string name_addition;
 
   if (simulation)
@@ -456,22 +460,21 @@ base_learner* VW::reductions::cs_active_setup(VW::setup_base_i& stack_builder)
   }
 
   size_t ws = data->num_classes;
-  auto* l = make_reduction_learner(std::move(data), as_singleline(stack_builder.setup_base_learner()), learn_ptr,
+  auto l = make_reduction_learner(std::move(data), require_singleline(stack_builder.setup_base_learner()), learn_ptr,
       predict_ptr, stack_builder.get_setupfn_name(cs_active_setup) + name_addition)
-                .set_params_per_weight(ws)
-                .set_learn_returns_prediction(true)
-                .set_input_prediction_type(VW::prediction_type_t::SCALAR)
-                .set_output_prediction_type(VW::prediction_type_t::ACTIVE_MULTICLASS)
-                .set_input_label_type(VW::label_type_t::CS)
-                .set_output_label_type(VW::label_type_t::SIMPLE)
-                .set_output_example_prediction(output_example_prediction_cs_active)
-                .set_print_update(print_update_cs_active)
-                .set_update_stats(update_stats_cs_active)
-                .build();
+               .set_params_per_weight(ws)
+               .set_learn_returns_prediction(true)
+               .set_input_prediction_type(VW::prediction_type_t::SCALAR)
+               .set_output_prediction_type(VW::prediction_type_t::ACTIVE_MULTICLASS)
+               .set_input_label_type(VW::label_type_t::CS)
+               .set_output_label_type(VW::label_type_t::SIMPLE)
+               .set_output_example_prediction(output_example_prediction_cs_active)
+               .set_print_update(print_update_cs_active)
+               .set_update_stats(update_stats_cs_active)
+               .build();
 
   // Label parser set to cost sensitive label parser
   all.example_parser->lbl_parser = VW::cs_label_parser_global;
-  base_learner* b = make_base(*l);
-  all.cost_sensitive = b;
-  return b;
+
+  return l;
 }
