@@ -110,7 +110,7 @@ void unsubtract_example(VW::example* ec, VW::io::logger& logger)
   ec->indices.pop_back();
 }
 
-void make_single_prediction(ldf& data, single_learner& base, VW::example& ec)
+void make_single_prediction(ldf& data, learner& base, VW::example& ec)
 {
   uint64_t old_offset = ec.ft_offset;
 
@@ -154,7 +154,7 @@ bool test_ldf_sequence(const VW::multi_ex& ec_seq, VW::io::logger& logger)
   return is_test;
 }
 
-void do_actual_learning_wap(ldf& data, single_learner& base, VW::multi_ex& ec_seq)
+void do_actual_learning_wap(ldf& data, learner& base, VW::multi_ex& ec_seq)
 {
   VW_DBG(ec_seq) << "do_actual_learning_wap()" << std::endl;
 
@@ -226,7 +226,7 @@ void do_actual_learning_wap(ldf& data, single_learner& base, VW::multi_ex& ec_se
   }
 }
 
-void do_actual_learning_oaa(ldf& data, single_learner& base, VW::multi_ex& ec_seq)
+void do_actual_learning_oaa(ldf& data, learner& base, VW::multi_ex& ec_seq)
 {
   VW_DBG(ec_seq) << "do_actual_learning_oaa()" << std::endl;
 
@@ -297,7 +297,7 @@ void do_actual_learning_oaa(ldf& data, single_learner& base, VW::multi_ex& ec_se
  * 2) verify no labels in the middle of data
  * 3) learn_or_predict(data) with rest
  */
-void learn_csoaa_ldf(ldf& data, single_learner& base, VW::multi_ex& ec_seq_all)
+void learn_csoaa_ldf(ldf& data, learner& base, VW::multi_ex& ec_seq_all)
 {
   if (ec_seq_all.empty())
   {
@@ -340,7 +340,7 @@ void convert_to_probabilities(const VW::multi_ex& ec_seq, VW::v_array<float>& pr
  * 2) verify no labels in the middle of data
  * 3) learn_or_predict(data) with rest
  */
-void predict_csoaa_ldf(ldf& data, single_learner& base, VW::multi_ex& ec_seq_all)
+void predict_csoaa_ldf(ldf& data, learner& base, VW::multi_ex& ec_seq_all)
 {
   if (ec_seq_all.empty())
   {
@@ -370,7 +370,7 @@ void predict_csoaa_ldf(ldf& data, single_learner& base, VW::multi_ex& ec_seq_all
   }
 }
 
-void predict_csoaa_ldf_probabilities(ldf& data, single_learner& base, VW::multi_ex& ec_seq_all)
+void predict_csoaa_ldf_probabilities(ldf& data, learner& base, VW::multi_ex& ec_seq_all)
 {
   if (ec_seq_all.empty())
   {
@@ -390,7 +390,7 @@ void predict_csoaa_ldf_probabilities(ldf& data, single_learner& base, VW::multi_
  * 2) verify no labels in the middle of data
  * 3) learn_or_predict(data) with rest
  */
-void predict_csoaa_ldf_rank(ldf& data, single_learner& base, VW::multi_ex& ec_seq_all)
+void predict_csoaa_ldf_rank(ldf& data, learner& base, VW::multi_ex& ec_seq_all)
 {
   data.ft_offset = ec_seq_all[0]->ft_offset;
   if (ec_seq_all.empty())
@@ -669,7 +669,7 @@ void print_update_csoaa_ldf_multiclass(VW::workspace& all, VW::shared_data& /* s
 }
 }  // namespace
 
-base_learner* VW::reductions::csldf_setup(VW::setup_base_i& stack_builder)
+std::shared_ptr<VW::LEARNER::learner> VW::reductions::csldf_setup(VW::setup_base_i& stack_builder)
 {
   options_i& options = *stack_builder.get_options();
   VW::workspace& all = *stack_builder.get_all_pointer();
@@ -755,7 +755,7 @@ base_learner* VW::reductions::csldf_setup(VW::setup_base_i& stack_builder)
   ld->label_features.max_load_factor(0.25);
   ld->label_features.reserve(256);
 
-  single_learner* base = as_singleline(stack_builder.setup_base_learner());
+  auto base = require_singleline(stack_builder.setup_base_learner());
   VW::learner_update_stats_func<ldf, VW::multi_ex>* update_stats_func = nullptr;
   VW::learner_output_example_prediction_func<ldf, VW::multi_ex>* output_example_prediction_func = nullptr;
   VW::learner_print_update_func<ldf, VW::multi_ex>* print_update_func = nullptr;
@@ -763,7 +763,7 @@ base_learner* VW::reductions::csldf_setup(VW::setup_base_i& stack_builder)
   std::string name = stack_builder.get_setupfn_name(csldf_setup);
   std::string name_addition;
   VW::prediction_type_t pred_type;
-  void (*pred_ptr)(ldf&, single_learner&, VW::multi_ex&);
+  void (*pred_ptr)(ldf&, learner&, VW::multi_ex&);
   if (ld->rank)
   {
     name_addition = "-rank";
@@ -792,18 +792,18 @@ base_learner* VW::reductions::csldf_setup(VW::setup_base_i& stack_builder)
     print_update_func = print_update_csoaa_ldf_multiclass;
   }
 
-  auto* l = make_reduction_learner(std::move(ld), base, learn_csoaa_ldf, pred_ptr, name + name_addition)
-                .set_end_pass(end_pass)
-                .set_input_label_type(VW::label_type_t::CS)
-                .set_output_label_type(VW::label_type_t::SIMPLE)
-                .set_input_prediction_type(VW::prediction_type_t::SCALAR)
-                .set_output_prediction_type(pred_type)
-                .set_update_stats(update_stats_func)
-                .set_output_example_prediction(output_example_prediction_func)
-                .set_print_update(print_update_func)
-                .build();
+  auto l = make_reduction_learner(std::move(ld), base, learn_csoaa_ldf, pred_ptr, name + name_addition)
+               .set_end_pass(end_pass)
+               .set_input_label_type(VW::label_type_t::CS)
+               .set_output_label_type(VW::label_type_t::SIMPLE)
+               .set_input_prediction_type(VW::prediction_type_t::SCALAR)
+               .set_output_prediction_type(pred_type)
+               .set_update_stats(update_stats_func)
+               .set_output_example_prediction(output_example_prediction_func)
+               .set_print_update(print_update_func)
+               .build();
 
   all.example_parser->lbl_parser = VW::cs_label_parser_global;
-  all.cost_sensitive = make_base(*l);
-  return all.cost_sensitive;
+
+  return l;
 }
