@@ -82,7 +82,7 @@ inline void predict_with_confidence(uncertainty& d, const float fx, float& fw)
   float uncertain = ((d.b.data.ftrl_beta + sqrtf_ng2) / d.b.data.ftrl_alpha + d.b.data.l2_lambda);
   d.score += (1 / uncertain) * sign(fx);
 }
-float sensitivity(ftrl& b, base_learner& /* base */, VW::example& ec)
+float sensitivity(ftrl& b, VW::example& ec)
 {
   uncertainty uncetain(b);
   VW::foreach_feature<uncertainty, predict_with_confidence>(*(b.all), ec, uncetain);
@@ -90,18 +90,18 @@ float sensitivity(ftrl& b, base_learner& /* base */, VW::example& ec)
 }
 
 template <bool audit>
-void predict(ftrl& b, base_learner&, VW::example& ec)
+void predict(ftrl& b, VW::example& ec)
 {
   size_t num_features_from_interactions = 0;
   ec.partial_prediction = VW::inline_predict(*b.all, ec, num_features_from_interactions);
   ec.num_features_from_interactions = num_features_from_interactions;
-  ec.pred.scalar = VW::details::finalize_prediction(b.all->sd, b.all->logger, ec.partial_prediction);
+  ec.pred.scalar = VW::details::finalize_prediction(*b.all->sd, b.all->logger, ec.partial_prediction);
   if (audit) { VW::details::print_audit_features(*(b.all), ec); }
 }
 
 template <bool audit>
-void multipredict(ftrl& b, base_learner&, VW::example& ec, size_t count, size_t step, VW::polyprediction* pred,
-    bool finalize_predictions)
+void multipredict(
+    ftrl& b, VW::example& ec, size_t count, size_t step, VW::polyprediction* pred, bool finalize_predictions)
 {
   VW::workspace& all = *b.all;
   for (size_t c = 0; c < count; c++)
@@ -133,7 +133,7 @@ void multipredict(ftrl& b, base_learner&, VW::example& ec, size_t count, size_t
   {
     for (size_t c = 0; c < count; c++)
     {
-      pred[c].scalar = VW::details::finalize_prediction(all.sd, all.logger, pred[c].scalar);
+      pred[c].scalar = VW::details::finalize_prediction(*all.sd, all.logger, pred[c].scalar);
     }
   }
   if (audit)
@@ -249,7 +249,7 @@ void inner_coin_betting_update_after_prediction(ftrl_update_data& d, float x, fl
   w[W_XT] /= d.average_squared_norm_x;
 }
 
-void coin_betting_predict(ftrl& b, base_learner&, VW::example& ec)
+void coin_betting_predict(ftrl& b, VW::example& ec)
 {
   b.data.predict = 0;
   b.data.normalized_squared_norm_x = 0;
@@ -264,10 +264,10 @@ void coin_betting_predict(ftrl& b, base_learner&, VW::example& ec)
 
   ec.partial_prediction = b.data.predict / b.data.average_squared_norm_x;
 
-  ec.pred.scalar = VW::details::finalize_prediction(b.all->sd, b.all->logger, ec.partial_prediction);
+  ec.pred.scalar = VW::details::finalize_prediction(*b.all->sd, b.all->logger, ec.partial_prediction);
 }
 
-void update_state_and_predict_pistol(ftrl& b, base_learner&, VW::example& ec)
+void update_state_and_predict_pistol(ftrl& b, VW::example& ec)
 {
   b.data.predict = 0;
 
@@ -277,54 +277,52 @@ void update_state_and_predict_pistol(ftrl& b, base_learner&, VW::example& ec)
   ec.num_features_from_interactions = num_features_from_interactions;
 
   ec.partial_prediction = b.data.predict;
-  ec.pred.scalar = VW::details::finalize_prediction(b.all->sd, b.all->logger, ec.partial_prediction);
+  ec.pred.scalar = VW::details::finalize_prediction(*b.all->sd, b.all->logger, ec.partial_prediction);
 }
 
 void update_after_prediction_proximal(ftrl& b, VW::example& ec)
 {
-  b.data.update = b.all->loss->first_derivative(b.all->sd, ec.pred.scalar, ec.l.simple.label) * ec.weight;
+  b.data.update = b.all->loss->first_derivative(b.all->sd.get(), ec.pred.scalar, ec.l.simple.label) * ec.weight;
   VW::foreach_feature<ftrl_update_data, inner_update_proximal>(*b.all, ec, b.data);
 }
 
 void update_after_prediction_pistol(ftrl& b, VW::example& ec)
 {
-  b.data.update = b.all->loss->first_derivative(b.all->sd, ec.pred.scalar, ec.l.simple.label) * ec.weight;
+  b.data.update = b.all->loss->first_derivative(b.all->sd.get(), ec.pred.scalar, ec.l.simple.label) * ec.weight;
   VW::foreach_feature<ftrl_update_data, inner_update_pistol_post>(*b.all, ec, b.data);
 }
 
 void coin_betting_update_after_prediction(ftrl& b, VW::example& ec)
 {
-  b.data.update = b.all->loss->first_derivative(b.all->sd, ec.pred.scalar, ec.l.simple.label) * ec.weight;
+  b.data.update = b.all->loss->first_derivative(b.all->sd.get(), ec.pred.scalar, ec.l.simple.label) * ec.weight;
   VW::foreach_feature<ftrl_update_data, inner_coin_betting_update_after_prediction>(*b.all, ec, b.data);
 }
 
-// NO_SANITIZE_UNDEFINED needed in learn functions because
-// base_learner& base might be a reference created from nullptr
 template <bool audit>
-void NO_SANITIZE_UNDEFINED learn_proximal(ftrl& a, base_learner& base, VW::example& ec)
+void learn_proximal(ftrl& a, VW::example& ec)
 {
   // predict with confidence
-  predict<audit>(a, base, ec);
+  predict<audit>(a, ec);
 
   // update state based on the prediction
   update_after_prediction_proximal(a, ec);
 }
 
 template <bool audit>
-void NO_SANITIZE_UNDEFINED learn_pistol(ftrl& a, base_learner& base, VW::example& ec)
+void learn_pistol(ftrl& a, VW::example& ec)
 {
   // update state based on the example and predict
-  update_state_and_predict_pistol(a, base, ec);
+  update_state_and_predict_pistol(a, ec);
   if (audit) { VW::details::print_audit_features(*(a.all), ec); }
   // update state based on the prediction
   update_after_prediction_pistol(a, ec);
 }
 
 template <bool audit>
-void NO_SANITIZE_UNDEFINED learn_coin_betting(ftrl& a, base_learner& base, VW::example& ec)
+void learn_coin_betting(ftrl& a, VW::example& ec)
 {
   // update state based on the example and predict
-  coin_betting_predict(a, base, ec);
+  coin_betting_predict(a, ec);
   if (audit) { VW::details::print_audit_features(*(a.all), ec); }
   // update state based on the prediction
   coin_betting_update_after_prediction(a, ec);
@@ -371,7 +369,7 @@ void end_pass(ftrl& g)
 }
 }  // namespace
 
-base_learner* VW::reductions::ftrl_setup(VW::setup_base_i& stack_builder)
+std::shared_ptr<VW::LEARNER::learner> VW::reductions::ftrl_setup(VW::setup_base_i& stack_builder)
 {
   options_i& options = *stack_builder.get_options();
   VW::workspace& all = *stack_builder.get_all_pointer();
@@ -422,7 +420,7 @@ base_learner* VW::reductions::ftrl_setup(VW::setup_base_i& stack_builder)
   b->total_weight = 0;
 
   std::string algorithm_name;
-  void (*learn_ptr)(ftrl&, base_learner&, VW::example&) = nullptr;
+  void (*learn_ptr)(ftrl&, VW::example&) = nullptr;
   bool learn_returns_prediction = false;
 
   // Defaults that are specific to the mode that was chosen.
@@ -479,7 +477,7 @@ base_learner* VW::reductions::ftrl_setup(VW::setup_base_i& stack_builder)
   auto multipredict_ptr = (all.audit || all.hash_inv) ? multipredict<true> : multipredict<false>;
   std::string name_addition = (all.audit || all.hash_inv) ? "-audit" : "";
 
-  auto l = VW::LEARNER::make_base_learner(std::move(b), learn_ptr, predict_ptr,
+  auto l = VW::LEARNER::make_bottom_learner(std::move(b), learn_ptr, predict_ptr,
       stack_builder.get_setupfn_name(ftrl_setup) + "-" + algorithm_name + name_addition, VW::prediction_type_t::SCALAR,
       VW::label_type_t::SIMPLE)
                .set_learn_returns_prediction(learn_returns_prediction)
@@ -492,5 +490,5 @@ base_learner* VW::reductions::ftrl_setup(VW::setup_base_i& stack_builder)
                .set_update_stats(VW::details::update_stats_simple_label<ftrl>)
                .set_print_update(VW::details::print_update_simple_label<ftrl>)
                .build();
-  return make_base(*l);
+  return l;
 }
\ No newline at end of file
