@@ -74,15 +74,18 @@ void merge_weights_with_save_resume(size_t length,
     for (size_t i = 0; i < length; i++) { adaptive_totals[i] += (&(this_model[i << weights.stride_shift()]))[1]; }
   }
 
-  for (size_t i = 0; i < source.size(); i++)
+  std::vector<VW::dense_parameters> weight_copies;
+  weight_copies.reserve(source.size());
+  for (auto i : source)
   {
-    VW::details::do_weighting(output_workspace.normalized_idx, length, adaptive_totals.data(), weights);
+    // There is no copy constructor for weights, so we have to copy manually.
+    weight_copies.emplace_back(VW::dense_parameters::deep_copy(i));
+    VW::details::do_weighting(output_workspace.normalized_idx, length, adaptive_totals.data(), weight_copies.back());
   }
 
   // Weights have already been reweighted, so just accumulate.
-  for (const auto& model_num : source)
+  for (const auto& this_source : weight_copies)
   {
-    const auto& this_source = model_num.get();
     // Intentionally add irrespective of stride.
     const auto full_weights_size = length << weights.stride_shift();
     for (uint64_t i = 0; i < full_weights_size; i++) { weights[i] += this_source[i]; }
@@ -242,7 +245,8 @@ void end_pass(VW::reductions::gd& g)
 }
 
 void merge(const std::vector<float>& per_model_weighting, const std::vector<const VW::workspace*>& all_workspaces,
-    const std::vector<VW::reductions::gd*>& all_data, VW::workspace& output_workspace, VW::reductions::gd& output_data)
+    const std::vector<const VW::reductions::gd*>& all_data, VW::workspace& output_workspace,
+    VW::reductions::gd& output_data)
 {
   const size_t length = static_cast<size_t>(1) << output_workspace.num_bits;
 
@@ -281,8 +285,8 @@ void merge(const std::vector<float>& per_model_weighting, const std::vector<cons
   }
 }
 
-void add(const VW::workspace& ws1, const VW::reductions::gd& data1, const VW::workspace& ws2, VW::reductions::gd& data2,
-    VW::workspace& ws_out, VW::reductions::gd& data_out)
+void add(const VW::workspace& ws1, const VW::reductions::gd& data1, const VW::workspace& ws2,
+    const VW::reductions::gd& data2, VW::workspace& ws_out, VW::reductions::gd& data_out)
 {
   const size_t length = static_cast<size_t>(1) << ws_out.num_bits;
   if (ws_out.weights.sparse)
@@ -303,7 +307,7 @@ void add(const VW::workspace& ws1, const VW::reductions::gd& data1, const VW::wo
 }
 
 void subtract(const VW::workspace& ws1, const VW::reductions::gd& data1, const VW::workspace& ws2,
-    VW::reductions::gd& data2, VW::workspace& ws_out, VW::reductions::gd& data_out)
+    const VW::reductions::gd& data2, VW::workspace& ws_out, VW::reductions::gd& data_out)
 {
   const size_t length = static_cast<size_t>(1) << ws_out.num_bits;
   if (ws_out.weights.sparse)
@@ -462,16 +466,16 @@ void VW::details::print_audit_features(VW::workspace& all, VW::example& ec)
   print_features(all, ec);
 }
 
-float VW::details::finalize_prediction(VW::shared_data* sd, VW::io::logger& logger, float ret)
+float VW::details::finalize_prediction(VW::shared_data& sd, VW::io::logger& logger, float ret)
 {
   if (std::isnan(ret))
   {
     ret = 0.;
-    logger.err_warn("NAN prediction in example {0}, forcing {1}", sd->example_number + 1, ret);
+    logger.err_warn("NAN prediction in example {0}, forcing {1}", sd.example_number + 1, ret);
     return ret;
   }
-  if (ret > sd->max_label) { return sd->max_label; }
-  if (ret < sd->min_label) { return sd->min_label; }
+  if (ret > sd.max_label) { return sd.max_label; }
+  if (ret < sd.min_label) { return sd.min_label; }
   return ret;
 }
 
@@ -498,7 +502,7 @@ inline float trunc_predict(VW::workspace& all, VW::example& ec, double gravity,
 }
 
 template <bool l1, bool audit>
-void predict(VW::reductions::gd& g, base_learner&, VW::example& ec)
+void predict(VW::reductions::gd& g, VW::example& ec)
 {
   VW_DBG(ec) << "gd.predict(): ex#=" << ec.example_counter << ", offset=" << ec.ft_offset << std::endl;
 
@@ -509,7 +513,7 @@ void predict(VW::reductions::gd& g, base_learner&, VW::example& ec)
 
   ec.num_features_from_interactions = num_interacted_features;
   ec.partial_prediction *= static_cast<float>(all.sd->contraction);
-  ec.pred.scalar = VW::details::finalize_prediction(all.sd, all.logger, ec.partial_prediction);
+  ec.pred.scalar = VW::details::finalize_prediction(*all.sd, all.logger, ec.partial_prediction);
 
   VW_DBG(ec) << "gd: predict() " << VW::debug::scalar_pred_to_string(ec) << VW::debug::features_to_string(ec)
              << std::endl;
@@ -528,8 +532,8 @@ inline void vec_add_trunc_multipredict(VW::details::multipredict_info<T>& mp, co
 }
 
 template <bool l1, bool audit>
-void multipredict(VW::reductions::gd& g, base_learner&, VW::example& ec, size_t count, size_t step,
-    VW::polyprediction* pred, bool finalize_predictions)
+void multipredict(VW::reductions::gd& g, VW::example& ec, size_t count, size_t step, VW::polyprediction* pred,
+    bool finalize_predictions)
 {
   VW::workspace& all = *g.all;
   for (size_t c = 0; c < count; c++)
@@ -579,7 +583,7 @@ void multipredict(VW::reductions::gd& g, base_learner&, VW::example& ec, size_t
   {
     for (size_t c = 0; c < count; c++)
     {
-      pred[c].scalar = VW::details::finalize_prediction(all.sd, all.logger, pred[c].scalar);
+      pred[c].scalar = VW::details::finalize_prediction(*all.sd, all.logger, pred[c].scalar);
     }
   }
   if (audit)
@@ -762,7 +766,7 @@ float get_scale(VW::reductions::gd& g, VW::example& /* ec */, float weight)
 }
 
 template <bool sqrt_rate, bool feature_mask_off, bool adax, size_t adaptive, size_t normalized, size_t spare>
-float sensitivity(VW::reductions::gd& g, base_learner& /* base */, VW::example& ec)
+float sensitivity(VW::reductions::gd& g, VW::example& ec)
 {
   return get_scale<adaptive>(g, ec, 1.) *
       sensitivity<sqrt_rate, feature_mask_off, adax, adaptive, normalized, spare, true>(g, ec);
@@ -778,7 +782,7 @@ float compute_update(VW::reductions::gd& g, VW::example& ec)
 
   float update = 0.;
   ec.updated_prediction = ec.pred.scalar;
-  if (all.loss->get_loss(all.sd, ec.pred.scalar, ld.label) > 0.)
+  if (all.loss->get_loss(all.sd.get(), ec.pred.scalar, ld.label) > 0.)
   {
     float pred_per_update = sensitivity<sqrt_rate, feature_mask_off, adax, adaptive, normalized, spare, false>(g, ec);
     float update_scale = get_scale<adaptive>(g, ec, ec.weight);
@@ -789,7 +793,7 @@ float compute_update(VW::reductions::gd& g, VW::example& ec)
 
     if (all.reg_mode && std::fabs(update) > 1e-8)
     {
-      double dev1 = all.loss->first_derivative(all.sd, ec.pred.scalar, ld.label);
+      double dev1 = all.loss->first_derivative(all.sd.get(), ec.pred.scalar, ld.label);
       double eta_bar = (fabs(dev1) > 1e-8) ? (-update / dev1) : 0.0;
       if (fabs(dev1) > 1e-8) { all.sd->contraction *= (1. - all.l2_lambda * eta_bar); }
       update /= static_cast<float>(all.sd->contraction);
@@ -810,7 +814,7 @@ float compute_update(VW::reductions::gd& g, VW::example& ec)
 
 template <bool sparse_l2, bool invariant, bool sqrt_rate, bool feature_mask_off, bool adax, size_t adaptive,
     size_t normalized, size_t spare>
-void update(VW::reductions::gd& g, base_learner&, VW::example& ec)
+void update(VW::reductions::gd& g, VW::example& ec)
 {
   // invariant: not a test label, importance weight > 0
   float update;
@@ -826,17 +830,15 @@ void update(VW::reductions::gd& g, base_learner&, VW::example& ec)
   }
 }
 
-// NO_SANITIZE_UNDEFINED needed in learn functions because
-// base_learner& base might be a reference created from nullptr
 template <bool sparse_l2, bool invariant, bool sqrt_rate, bool feature_mask_off, bool adax, size_t adaptive,
     size_t normalized, size_t spare>
-void NO_SANITIZE_UNDEFINED learn(VW::reductions::gd& g, base_learner& base, VW::example& ec)
+void learn(VW::reductions::gd& g, VW::example& ec)
 {
   // invariant: not a test label, importance weight > 0
   assert(ec.l.simple.label != FLT_MAX);
   assert(ec.weight > 0.);
-  g.predict(g, base, ec);
-  update<sparse_l2, invariant, sqrt_rate, feature_mask_off, adax, adaptive, normalized, spare>(g, base, ec);
+  g.predict(g, ec);
+  update<sparse_l2, invariant, sqrt_rate, feature_mask_off, adax, adaptive, normalized, spare>(g, ec);
 }
 
 size_t write_index(VW::io_buf& model_file, std::stringstream& msg, bool text, uint32_t num_bits, uint64_t i)
@@ -1360,7 +1362,7 @@ uint64_t ceil_log_2(uint64_t v)
 
 }  // namespace
 
-base_learner* VW::reductions::gd_setup(VW::setup_base_i& stack_builder)
+std::shared_ptr<VW::LEARNER::learner> VW::reductions::gd_setup(VW::setup_base_i& stack_builder)
 {
   options_i& options = *stack_builder.get_options();
   VW::workspace& all = *stack_builder.get_all_pointer();
@@ -1502,22 +1504,21 @@ base_learner* VW::reductions::gd_setup(VW::setup_base_i& stack_builder)
   all.weights.stride_shift(static_cast<uint32_t>(::ceil_log_2(stride - 1)));
 
   auto* bare = g.get();
-  learner<VW::reductions::gd, VW::example>* l =
-      make_base_learner(std::move(g), g->learn, bare->predict, stack_builder.get_setupfn_name(gd_setup),
-          VW::prediction_type_t::SCALAR, VW::label_type_t::SIMPLE)
-          .set_learn_returns_prediction(true)
-          .set_params_per_weight(VW::details::UINT64_ONE << all.weights.stride_shift())
-          .set_sensitivity(bare->sensitivity)
-          .set_multipredict(bare->multipredict)
-          .set_update(bare->update)
-          .set_save_load(::save_load)
-          .set_end_pass(::end_pass)
-          .set_merge_with_all(::merge)
-          .set_add_with_all(::add)
-          .set_subtract_with_all(::subtract)
-          .set_output_example_prediction(VW::details::output_example_prediction_simple_label<VW::reductions::gd>)
-          .set_update_stats(VW::details::update_stats_simple_label<VW::reductions::gd>)
-          .set_print_update(VW::details::print_update_simple_label<VW::reductions::gd>)
-          .build();
-  return make_base(*l);
+  auto l = make_bottom_learner(std::move(g), g->learn, bare->predict, stack_builder.get_setupfn_name(gd_setup),
+      VW::prediction_type_t::SCALAR, VW::label_type_t::SIMPLE)
+               .set_learn_returns_prediction(true)
+               .set_params_per_weight(VW::details::UINT64_ONE << all.weights.stride_shift())
+               .set_sensitivity(bare->sensitivity)
+               .set_multipredict(bare->multipredict)
+               .set_update(bare->update)
+               .set_save_load(::save_load)
+               .set_end_pass(::end_pass)
+               .set_merge_with_all(::merge)
+               .set_add_with_all(::add)
+               .set_subtract_with_all(::subtract)
+               .set_output_example_prediction(VW::details::output_example_prediction_simple_label<VW::reductions::gd>)
+               .set_update_stats(VW::details::update_stats_simple_label<VW::reductions::gd>)
+               .set_print_update(VW::details::print_update_simple_label<VW::reductions::gd>)
+               .build();
+  return l;
 }
