@@ -52,7 +52,7 @@ public:
   }
 };
 
-void learn_randomized(oaa& o, VW::LEARNER::single_learner& base, VW::example& ec)
+void learn_randomized(oaa& o, VW::LEARNER::learner& base, VW::example& ec)
 {
   // Update indexing
   if (o.indexing == 2 && ec.l.multi.label == 0)
@@ -116,7 +116,7 @@ void learn_randomized(oaa& o, VW::LEARNER::single_learner& base, VW::example& ec
 }
 
 template <bool print_all, bool scores, bool probabilities>
-void learn(oaa& o, VW::LEARNER::single_learner& base, VW::example& ec)
+void learn(oaa& o, VW::LEARNER::learner& base, VW::example& ec)
 {
   // Update indexing
   if (o.indexing == 2 && ec.l.multi.label == 0)
@@ -166,7 +166,7 @@ void learn(oaa& o, VW::LEARNER::single_learner& base, VW::example& ec)
 }
 
 template <bool print_all, bool scores, bool probabilities>
-void predict(oaa& o, VW::LEARNER::single_learner& base, VW::example& ec)
+void predict(oaa& o, VW::LEARNER::learner& base, VW::example& ec)
 {
   // The predictions are either an array of scores or a single
   // class id of a multiclass label
@@ -322,7 +322,7 @@ void output_example_prediction_oaa(
 }
 }  // namespace
 
-VW::LEARNER::base_learner* VW::reductions::oaa_setup(VW::setup_base_i& stack_builder)
+std::shared_ptr<VW::LEARNER::learner> VW::reductions::oaa_setup(VW::setup_base_i& stack_builder)
 {
   options_i& options = *stack_builder.get_options();
   VW::workspace& all = *stack_builder.get_all_pointer();
@@ -375,9 +375,9 @@ VW::LEARNER::base_learner* VW::reductions::oaa_setup(VW::setup_base_i& stack_bui
 
   oaa* data_ptr = data.get();
   uint64_t k_value = data->k;
-  auto* base = as_singleline(stack_builder.setup_base_learner());
-  void (*learn_ptr)(oaa&, VW::LEARNER::single_learner&, VW::example&) = nullptr;
-  void (*pred_ptr)(oaa&, VW::LEARNER::single_learner&, VW::example&) = nullptr;
+  auto base = require_singleline(stack_builder.setup_base_learner());
+  void (*learn_ptr)(oaa&, VW::LEARNER::learner&, VW::example&) = nullptr;
+  void (*pred_ptr)(oaa&, VW::LEARNER::learner&, VW::example&) = nullptr;
   std::string name_addition;
   VW::prediction_type_t pred_type;
 
@@ -453,17 +453,17 @@ VW::LEARNER::base_learner* VW::reductions::oaa_setup(VW::setup_base_i& stack_bui
     };
   }
 
-  auto* l = make_reduction_learner(
+  auto l = make_reduction_learner(
       std::move(data), base, learn_ptr, pred_ptr, stack_builder.get_setupfn_name(oaa_setup) + name_addition)
-                .set_params_per_weight(k_value)
-                .set_input_label_type(VW::label_type_t::MULTICLASS)
-                .set_output_label_type(VW::label_type_t::SIMPLE)
-                .set_input_prediction_type(VW::prediction_type_t::SCALAR)
-                .set_output_prediction_type(pred_type)
-                .set_update_stats(update_stats_func)
-                .set_output_example_prediction(output_example_prediction_func)
-                .set_print_update(print_update_func)
-                .build();
+               .set_params_per_weight(k_value)
+               .set_input_label_type(VW::label_type_t::MULTICLASS)
+               .set_output_label_type(VW::label_type_t::SIMPLE)
+               .set_input_prediction_type(VW::prediction_type_t::SCALAR)
+               .set_output_prediction_type(pred_type)
+               .set_update_stats(update_stats_func)
+               .set_output_example_prediction(output_example_prediction_func)
+               .set_print_update(print_update_func)
+               .build();
 
-  return make_base(*l);
+  return l;
 }
