@@ -46,7 +46,7 @@ TEST(Las, CheckMatricsWithLASRunsOK)
 
   vw->finish_example(examples);
 
-  auto metrics = vw->global_metrics.collect_metrics(vw->l);
+  auto metrics = vw->global_metrics.collect_metrics(vw->l.get());
   EXPECT_EQ(metrics.get_uint("cbea_labeled_ex"), 1);
   EXPECT_EQ(metrics.get_uint("cb_las_filtering_factor"), 5);
 }
@@ -71,14 +71,14 @@ TEST(Las, CheckAOSameActionsSameRepresentation)
     auto& vw = *vw_ptr;
 
     std::vector<std::string> e_r;
-    vw.l->get_enabled_reductions(e_r);
+    vw.l->get_enabled_learners(e_r);
     if (std::find(e_r.begin(), e_r.end(), "cb_explore_adf_large_action_space") == e_r.end())
     {
-      FAIL() << "cb_explore_adf_large_action_space not found in enabled reductions";
+      FAIL() << "cb_explore_adf_large_action_space not found in enabled learners";
     }
 
-    VW::LEARNER::multi_learner* learner =
-        as_multiline(vw.l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner =
+        require_multiline(vw.l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
 
     auto* action_space = (internal_action_space_op*)learner->get_internal_type_erased_data_pointer_test_use_only();
 
@@ -131,14 +131,14 @@ TEST(Las, CheckAOLinearCombinationOfActions)
     auto& vw = *vw_ptr;
 
     std::vector<std::string> e_r;
-    vw.l->get_enabled_reductions(e_r);
+    vw.l->get_enabled_learners(e_r);
     if (std::find(e_r.begin(), e_r.end(), "cb_explore_adf_large_action_space") == e_r.end())
     {
-      FAIL() << "cb_explore_adf_large_action_space not found in enabled reductions";
+      FAIL() << "cb_explore_adf_large_action_space not found in enabled learners";
     }
 
-    VW::LEARNER::multi_learner* learner =
-        as_multiline(vw.l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner =
+        require_multiline(vw.l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
 
     auto* action_space = (internal_action_space_op*)learner->get_internal_type_erased_data_pointer_test_use_only();
 
@@ -341,8 +341,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
 
     auto vw_scalar = VW::initialize(VW::make_unique<VW::config::options_cli>(vw_cmd));
 
-    VW::LEARNER::multi_learner* learner_scalar =
-        as_multiline(vw_scalar->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner_scalar =
+        require_multiline(vw_scalar->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space_scalar =
         (internal_action_space_op*)learner_scalar->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space_scalar, nullptr);
@@ -357,8 +357,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
     vw_cmd.push_back("--las_hint_explicit_simd");
     auto vw_simd = VW::initialize(VW::make_unique<VW::config::options_cli>(vw_cmd));
 
-    VW::LEARNER::multi_learner* learner_simd =
-        as_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner_simd =
+        require_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space_simd =
         (internal_action_space_op*)learner_simd->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space_simd, nullptr);
@@ -387,8 +387,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
 
     auto vw_scalar = VW::initialize(VW::make_unique<VW::config::options_cli>(vw_cmd));
 
-    VW::LEARNER::multi_learner* learner_scalar =
-        as_multiline(vw_scalar->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner_scalar =
+        require_multiline(vw_scalar->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space_scalar =
         (internal_action_space_op*)learner_scalar->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space_scalar, nullptr);
@@ -403,8 +403,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
     vw_cmd.push_back("--las_hint_explicit_simd");
     auto vw_simd = VW::initialize(VW::make_unique<VW::config::options_cli>(vw_cmd));
 
-    VW::LEARNER::multi_learner* learner_simd =
-        as_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner_simd =
+        require_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space_simd =
         (internal_action_space_op*)learner_simd->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space_simd, nullptr);
@@ -434,8 +434,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
 
     auto vw_scalar = VW::initialize(VW::make_unique<VW::config::options_cli>(vw_cmd));
 
-    VW::LEARNER::multi_learner* learner_scalar =
-        as_multiline(vw_scalar->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner_scalar =
+        require_multiline(vw_scalar->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space_scalar =
         (internal_action_space_op*)learner_scalar->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space_scalar, nullptr);
@@ -450,8 +450,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
     vw_cmd.push_back("--las_hint_explicit_simd");
     auto vw_simd = VW::initialize(VW::make_unique<VW::config::options_cli>(vw_cmd));
 
-    VW::LEARNER::multi_learner* learner_simd =
-        as_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner_simd =
+        require_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space_simd =
         (internal_action_space_op*)learner_simd->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space_simd, nullptr);
@@ -479,8 +479,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
     auto vw_simd = VW::initialize(vwtest::make_args(
         "--cb_explore_adf", "--large_action_space", "--quiet", "--cubic", ":::", "--las_hint_explicit_simd"));
 
-    VW::LEARNER::multi_learner* learner =
-        as_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner =
+        require_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space = (internal_action_space_op*)learner->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space, nullptr);
 
@@ -491,8 +491,8 @@ TEST(Las, ScalarAndSimdGenerateSamePredictions)
     auto vw_simd = VW::initialize(vwtest::make_args("--cb_explore_adf", "--large_action_space", "--quiet",
         "--experimental_full_name_interactions", "A|B", "--las_hint_explicit_simd"));
 
-    VW::LEARNER::multi_learner* learner =
-        as_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
+    VW::LEARNER::learner* learner =
+        require_multiline(vw_simd->l->get_learner_by_name_prefix("cb_explore_adf_large_action_space"));
     auto* action_space = (internal_action_space_op*)learner->get_internal_type_erased_data_pointer_test_use_only();
     EXPECT_NE(action_space, nullptr);
 
