@@ -438,16 +438,16 @@ TEST(ParseJson, DedupCb)
 
   // parse first dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_1.c_str(), action_1.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_1.c_str(), action_1.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_1, examples[0]);
 
   examples.clear();
 
   // parse second dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_2.c_str(), action_2.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_2.c_str(), action_2.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_2, examples[0]);
 
   examples.clear();
@@ -455,7 +455,7 @@ TEST(ParseJson, DedupCb)
   // parse json that includes dedup id's and re-use the examples from the dedup map instead of creating new ones
   examples.push_back(&VW::get_unused_example(vw.get()));
   VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(), json_deduped_text.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get(), &dedup_examples);
+      std::bind(VW::get_unused_example, vw.get()), &dedup_examples);
 
   EXPECT_EQ(examples.size(), 3);                       // shared example + 2 multi examples
   EXPECT_NE(examples[1], dedup_examples[dedup_id_1]);  // checking pointers
@@ -503,25 +503,24 @@ TEST(ParseJson, DedupCbMissingDedupId)
 
   // parse first dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_1.c_str(), action_1.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_1.c_str(), action_1.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_1, examples[0]);
 
   examples.clear();
 
   // parse second dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_2.c_str(), action_2.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_2.c_str(), action_2.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_2, examples[0]);
 
   examples.clear();
 
   // parse json that includes dedup id's and re-use the examples from the dedup map instead of creating new ones
   examples.push_back(&VW::get_unused_example(vw.get()));
-  EXPECT_THROW(
-      VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(),
-          json_deduped_text.length(), (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get(), &dedup_examples),
+  EXPECT_THROW(VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(),
+                   json_deduped_text.length(), std::bind(VW::get_unused_example, vw.get()), &dedup_examples),
       VW::vw_exception);
 
   for (auto* example : examples) { VW::finish_example(*vw, *example); }
@@ -570,16 +569,16 @@ TEST(ParseJson, DedupCcb)
 
   // parse first dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_1.c_str(), action_1.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_1.c_str(), action_1.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_1, examples[0]);
 
   examples.clear();
 
   // parse second dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_2.c_str(), action_2.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_2.c_str(), action_2.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_2, examples[0]);
 
   examples.clear();
@@ -587,7 +586,7 @@ TEST(ParseJson, DedupCcb)
   // parse json that includes dedup id's and re-use the examples from the dedup map instead of creating new ones
   examples.push_back(&VW::get_unused_example(vw.get()));
   VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(), json_deduped_text.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get(), &dedup_examples);
+      std::bind(VW::get_unused_example, vw.get()), &dedup_examples);
 
   EXPECT_EQ(examples.size(), 6);                       // shared example + 2 multi examples + 3 slots
   EXPECT_NE(examples[1], dedup_examples[dedup_id_1]);  // checking pointers
@@ -688,25 +687,24 @@ TEST(ParseJson, DedupCcbDedupIdMissing)
 
   // parse first dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_1.c_str(), action_1.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_1.c_str(), action_1.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_1, examples[0]);
 
   examples.clear();
 
   // parse second dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_2.c_str(), action_2.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_2.c_str(), action_2.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_2, examples[0]);
 
   examples.clear();
 
   // parse json that includes dedup id's and re-use the examples from the dedup map instead of creating new ones
   examples.push_back(&VW::get_unused_example(vw.get()));
-  EXPECT_THROW(
-      VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(),
-          json_deduped_text.length(), (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get(), &dedup_examples),
+  EXPECT_THROW(VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(),
+                   json_deduped_text.length(), std::bind(VW::get_unused_example, vw.get()), &dedup_examples),
       VW::vw_exception);
 
   for (auto* example : examples) { VW::finish_example(*vw, *example); }
@@ -735,16 +733,16 @@ TEST(ParseJson, DedupSlates)
 
   // parse first dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_1.c_str(), action_1.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_1.c_str(), action_1.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_1, examples[0]);
 
   examples.clear();
 
   // parse second dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_2.c_str(), action_2.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_2.c_str(), action_2.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_2, examples[0]);
 
   examples.clear();
@@ -752,7 +750,7 @@ TEST(ParseJson, DedupSlates)
   // parse json that includes dedup id's and re-use the examples from the dedup map instead of creating new ones
   examples.push_back(&VW::get_unused_example(vw.get()));
   VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(), json_deduped_text.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get(), &dedup_examples);
+      std::bind(VW::get_unused_example, vw.get()), &dedup_examples);
 
   EXPECT_EQ(examples.size(), 5);                       // shared example + 2 multi examples + 2 slots
   EXPECT_NE(examples[1], dedup_examples[dedup_id_1]);  // checking pointers
@@ -812,16 +810,16 @@ TEST(ParseJson, DedupSlatesDedupIdMissing)
 
   // parse first dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_1.c_str(), action_1.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_1.c_str(), action_1.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_1, examples[0]);
 
   examples.clear();
 
   // parse second dedup example and store it in dedup_examples map
   examples.push_back(&VW::get_unused_example(vw.get()));
-  VW::parsers::json::read_line_json<true>(*vw, examples, (char*)action_2.c_str(), action_2.length(),
-      (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get());
+  VW::parsers::json::read_line_json<true>(
+      *vw, examples, (char*)action_2.c_str(), action_2.length(), std::bind(VW::get_unused_example, vw.get()));
   dedup_examples.emplace(dedup_id_2, examples[0]);
 
   examples.clear();
@@ -829,9 +827,8 @@ TEST(ParseJson, DedupSlatesDedupIdMissing)
   // parse json that includes dedup id's and re-use the examples from the dedup map instead of creating new ones
   examples.push_back(&VW::get_unused_example(vw.get()));
 
-  EXPECT_THROW(
-      VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(),
-          json_deduped_text.length(), (VW::example_factory_t)&VW::get_unused_example, (void*)vw.get(), &dedup_examples),
+  EXPECT_THROW(VW::parsers::json::read_line_json<true>(*vw, examples, (char*)json_deduped_text.c_str(),
+                   json_deduped_text.length(), std::bind(VW::get_unused_example, vw.get()), &dedup_examples),
       VW::vw_exception);
 
   for (auto* example : examples) { VW::finish_example(*vw, *example); }
