@@ -2,16 +2,23 @@
 #include "vw/config/options_cli.h"
 #include "vw/core/array_parameters_dense.h"
 #include "vw/core/cache.h"
+#include "vw/core/cb.h"
 #include "vw/core/constant.h"
+#include "vw/core/cost_sensitive.h"
 #include "vw/core/example.h"
 #include "vw/core/global_data.h"
+#include "vw/core/guard.h"
+#include "vw/core/io_buf.h"
 #include "vw/core/label_type.h"
 #include "vw/core/learner.h"
 #include "vw/core/loss_functions.h"
 #include "vw/core/memory.h"
 #include "vw/core/merge.h"
+#include "vw/core/metric_sink.h"
+#include "vw/core/multiclass.h"
 #include "vw/core/object_pool.h"
 #include "vw/core/parse_example.h"
+#include "vw/core/parse_regressor.h"
 #include "vw/core/prediction_type.h"
 #include "vw/core/scope_exit.h"
 #include "vw/core/simple_label.h"
@@ -22,6 +29,7 @@
 #include "vw/json_parser/decision_service_utils.h"
 #include "vw/json_parser/parse_example_json.h"
 
+#include <pybind11/cast.h>
 #include <pybind11/numpy.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/pytypes.h>
@@ -31,10 +39,21 @@
 #include <iostream>
 #include <memory>
 #include <optional>
+#include <variant>
 
 #define STRINGIFY(x) #x
 #define MACRO_STRINGIFY(x) STRINGIFY(x)
 
+// helper type for the visitor #4
+template <class... Ts>
+struct overloaded : Ts...
+{
+  using Ts::operator()...;
+};
+// explicit deduction guide (not needed as of C++20)
+template <class... Ts>
+overloaded(Ts...) -> overloaded<Ts...>;
+
 namespace py = pybind11;
 namespace
 {
@@ -82,6 +101,9 @@ void clean_example(VW::example& ec)
 {
   for (auto& fs : ec) { fs.clear(); }
 
+  ec.pred = VW::polyprediction{};
+  ec.l = VW::polylabel{};
+  ec.ex_reduction_features.clear();
   ec.indices.clear();
   ec.tag.clear();
   ec.sorted = false;
@@ -93,7 +115,7 @@ void clean_example(VW::example& ec)
 
 std::shared_ptr<VW::example> get_example_from_pool()
 {
-  return std::shared_ptr<VW::example>(SHARED_EXAMPLE_POOL.get_object(),
+  return std::shared_ptr<VW::example>(SHARED_EXAMPLE_POOL.get_object().release(),
       [](VW::example* ptr)
       {
         clean_example(*ptr);
@@ -306,13 +328,9 @@ std::vector<std::shared_ptr<VW::example>> parse_dsjson_line(
   auto ex = get_example_from_pool();
 
   VW::multi_ex examples;
-  examples.push_back(SHARED_EXAMPLE_POOL.get_object());
+  examples.push_back(SHARED_EXAMPLE_POOL.get_object().release());
 
-  auto example_factory = [](void* context) -> VW::example&
-  {
-    auto* pool = static_cast<VW::object_pool<VW::example>*>(context);
-    return *pool->get_object();
-  };
+  auto example_factory = []() -> VW::example& { return *SHARED_EXAMPLE_POOL.get_object().release(); };
 
   VW::parsers::json::decision_service_interaction interaction;
   try
@@ -324,8 +342,18 @@ std::vector<std::shared_ptr<VW::example>> parse_dsjson_line(
 
     // Not using the copy_line param as there were parse issues caused. It is possible they are due to the fact the line
     // input does not necessarily have a null terminator.
-    bool result = VW::parsers::json::read_line_decision_service_json<false>(*workspace.workspace_ptr, examples,
-        owned_str.data(), owned_str.size(), false, example_factory, &SHARED_EXAMPLE_POOL, &interaction);
+
+    bool result;
+    if (workspace.workspace_ptr->audit || workspace.workspace_ptr->hash_inv)
+    {
+      result = VW::parsers::json::read_line_decision_service_json<true>(
+          *workspace.workspace_ptr, examples, owned_str.data(), owned_str.size(), false, example_factory, &interaction);
+    }
+    else
+    {
+      result = VW::parsers::json::read_line_decision_service_json<false>(
+          *workspace.workspace_ptr, examples, owned_str.data(), owned_str.size(), false, example_factory, &interaction);
+    }
 
     // Since we are using strict parse any errors should be surfaced via an exception.
     assert(result);
@@ -355,6 +383,61 @@ std::vector<std::shared_ptr<VW::example>> parse_dsjson_line(
   return result;
 }
 
+std::vector<std::shared_ptr<VW::example>> parse_json_line(
+    workspace_with_logger_contexts& workspace, std::string_view line)
+{
+  auto ex = get_example_from_pool();
+
+  VW::multi_ex examples;
+  examples.push_back(SHARED_EXAMPLE_POOL.get_object().release());
+
+  auto example_factory = []() -> VW::example& { return *SHARED_EXAMPLE_POOL.get_object().release(); };
+
+  VW::parsers::json::decision_service_interaction interaction;
+  try
+  {
+    // Must copy as the input is destructively parsed.
+    std::vector<char> owned_str;
+    owned_str.resize(line.size() + 1);
+    std::memcpy(owned_str.data(), line.data(), line.size());
+    owned_str[line.size()] = '\0';
+
+    if (workspace.workspace_ptr->audit || workspace.workspace_ptr->hash_inv)
+    {
+      VW::parsers::json::template read_line_json<true>(
+          *workspace.workspace_ptr, examples, owned_str.data(), owned_str.size(), example_factory);
+    }
+    else
+    {
+      VW::parsers::json::template read_line_json<false>(
+          *workspace.workspace_ptr, examples, owned_str.data(), owned_str.size(), example_factory);
+    }
+  }
+  catch (const VW::vw_exception& ex)
+  {
+    for (auto* ex : examples)
+    {
+      clean_example(*ex);
+      SHARED_EXAMPLE_POOL.return_object(ex);
+    }
+    throw;
+  }
+
+  std::vector<std::shared_ptr<VW::example>> result;
+  result.reserve(examples.size());
+  for (auto* ex : examples)
+  {
+    result.emplace_back(ex,
+        [](VW::example* ex)
+        {
+          clean_example(*ex);
+          SHARED_EXAMPLE_POOL.return_object(ex);
+        });
+  }
+
+  return result;
+}
+
 // Impl from VW
 void write_cache_header(workspace_with_logger_contexts& workspace, py::object file)
 {
@@ -414,25 +497,20 @@ void update_stats_recursive(VW::workspace& workspace, LearnerT& learner, Example
 
   // Finish example used to utilize the copy forwarding semantics.
   // Traverse until first hit to mimic this but with greater type safety.
-  auto* base = learner.get_learn_base();
+  auto* base = learner.get_base_learner();
   if (base != nullptr)
   {
     if (learner.is_multiline() != base->is_multiline())
     {
       THROW("Cannot forward update_stats call across multiline/singleline boundary.");
     }
-    if (base->is_multiline()) { as_multiline(base)->finish_example(workspace, (VW::multi_ex&)example); }
-    else { as_singleline(base)->finish_example(workspace, (VW::example&)example); }
+    base->finish_example(workspace, example);
   }
   else { THROW("No update_stats functions were registered in the stack."); }
 }
 
 void py_setup_example(VW::workspace& ws, VW::example& ex)
 {
-#ifndef NDEBUG
-  for (auto& fg : *ae) { assert(fg.validate_extents()); }
-#endif
-
   ex.partial_prediction = 0.;
   ex.num_features = 0;
   ex.reset_total_sum_feat_sq();
@@ -478,10 +556,45 @@ void py_setup_example(VW::workspace& ws, std::vector<VW::example*>& ex)
 
 void py_unsetup_example(VW::workspace& ws, VW::example& ex)
 {
-  // Reset these to avoid reuse issues.
+  // Reset these to avoid reuse issues, but make sure keep the label that was passed in.
   // This is wasteful from a memory perspective but important for correctness at
   // the moment.
-  ex.l = VW::polylabel{};
+  VW::polylabel replacement{};
+  switch (ws.l->get_input_label_type())
+  {
+    case VW::label_type_t::SIMPLE:
+      replacement.simple = std::move(ex.l.simple);
+      break;
+    case VW::label_type_t::CB:
+      replacement.cb = std::move(ex.l.cb);
+      break;
+    case VW::label_type_t::CB_EVAL:
+      replacement.cb_eval = std::move(ex.l.cb_eval);
+      break;
+    case VW::label_type_t::CS:
+      replacement.cs = std::move(ex.l.cs);
+      break;
+    case VW::label_type_t::MULTILABEL:
+      replacement.multilabels = std::move(ex.l.multilabels);
+      break;
+    case VW::label_type_t::MULTICLASS:
+      replacement.multi = std::move(ex.l.multi);
+      break;
+    case VW::label_type_t::CCB:
+      replacement.conditional_contextual_bandit = std::move(ex.l.conditional_contextual_bandit);
+      break;
+    case VW::label_type_t::SLATES:
+      replacement.slates = std::move(ex.l.slates);
+      break;
+    case VW::label_type_t::NOLABEL:
+      break;
+    case VW::label_type_t::CONTINUOUS:
+      replacement.cb_cont = std::move(ex.l.cb_cont);
+      break;
+    default:
+      THROW("Unknown label type encountered in py_unsetup_example");
+  }
+  ex.l = std::move(replacement);
   ex.pred = VW::polyprediction{};
 
   if (ws.add_constant)
@@ -584,6 +697,52 @@ struct dense_weight_holder
   size_t wpp;
   std::shared_ptr<VW::workspace> ws;
 };
+
+struct python_dict_writer : VW::metric_sink_visitor
+{
+  python_dict_writer(py::dict& dest_dict) : _dest_dict(dest_dict) {}
+  void int_metric(const std::string& key, uint64_t value) override { _dest_dict[key.c_str()] = value; }
+  void float_metric(const std::string& key, float value) override { _dest_dict[key.c_str()] = value; }
+  void string_metric(const std::string& key, const std::string& value) override { _dest_dict[key.c_str()] = value; }
+  void bool_metric(const std::string& key, bool value) override { _dest_dict[key.c_str()] = value; }
+  void sink_metric(const std::string& key, const VW::metric_sink& value) override
+  {
+    py::dict nested;
+    auto nested_py = python_dict_writer(nested);
+    value.visit(nested_py);
+    _dest_dict[key.c_str()] = nested;
+  }
+
+private:
+  py::dict& _dest_dict;
+};
+
+py::dict convert_metrics_to_dict(const VW::metric_sink& metrics)
+{
+  py::dict dictionary;
+  python_dict_writer writer(dictionary);
+  metrics.visit(writer);
+  return dictionary;
+}
+
+// Labels
+
+struct py_simple_label
+{
+  float label;
+  float weight;
+  float initial;
+};
+
+bool is_shared(const VW::cs_label& label)
+{
+  const auto& costs = label.costs;
+  if (costs.size() != 1) { return false; }
+  if (costs[0].class_index != 0) { return false; }
+  if (costs[0].x != -FLT_MAX) { return false; }
+  return true;
+}
+
 }  // namespace
 
 PYBIND11_MODULE(_core, m)
@@ -635,6 +794,211 @@ PYBIND11_MODULE(_core, m)
       .value("ActiveMulticlass", VW::prediction_type_t::ACTIVE_MULTICLASS)
       .value("NoPred", VW::prediction_type_t::NOPRED);
 
+  py::class_<py_simple_label>(m, "SimpleLabel")
+      .def(py::init(
+               [](float label, float weight, float initial)
+               {
+                 py_simple_label l;
+                 l.label = label;
+                 l.weight = weight;
+                 l.initial = initial;
+                 return l;
+               }),
+          R"docstring(
+    A label representing a simple regression problem.
+
+    Args:
+      label (float): The label.
+      weight (float): The weight of the example.
+      initial (float): The initial value of the prediction.
+)docstring",
+          py::arg("label"), py::arg("weight") = 1.f, py::arg("initial") = 0.f)
+      .def_readwrite("label", &py_simple_label::label, R"docstring(
+    The label.
+)docstring")
+      .def_readwrite("weight", &py_simple_label::weight, R"docstring(
+    The weight of this label.
+)docstring")
+      .def_readwrite("initial", &py_simple_label::initial, R"docstring(
+    The initial value of the prediction.
+)docstring");
+
+  py::class_<VW::multiclass_label>(m, "MulticlassLabel")
+      .def(py::init(
+               [](uint32_t label, float weight)
+               {
+                 VW::multiclass_label l;
+                 l.label = label;
+                 l.weight = weight;
+                 return l;
+               }),
+          R"docstring(
+    A label representing a multiclass classification problem.
+
+    Args:
+      label (int): The label.
+      weight (float): The weight of the example.
+)docstring",
+          py::arg("label"), py::arg("weight") = 1.f)
+      .def_readwrite("label", &VW::multiclass_label::label, R"docstring(
+    The class of this label.
+)docstring")
+      .def_readwrite("weight", &VW::multiclass_label::weight, R"docstring(
+    The weight of this label.
+)docstring");
+
+  py::class_<VW::cb_label>(m, "CBLabel")
+      .def(py::init(
+               [](std::optional<std::variant<std::tuple<float, float>, std::tuple<uint32_t, float, float>>> label_value,
+                   float weight, bool is_shared)
+               {
+                 auto label = std::make_unique<VW::cb_label>();
+                 label->weight = weight;
+                 if (is_shared)
+                 {
+                   if (label_value.has_value())
+                   {
+                     throw std::invalid_argument("Shared examples cannot have action, cost, or probability.");
+                   }
+                   // Shared examples have essentially a sentinel value as prob.
+                   label->costs.emplace_back(VW::cb_class(0.f, 0, -1.f));
+                   return label;
+                 }
+
+                 if (label_value.has_value())
+                 {
+                   std::visit(
+                       overloaded{
+                           [&](std::tuple<float, float> value)
+                           {
+                             auto [cost, probability] = value;
+                             label->costs.emplace_back(VW::cb_class(cost, 0, probability));
+                           },
+                           [&](std::tuple<uint32_t, float, float> value)
+                           {
+                             auto [action, cost, probability] = value;
+                             label->costs.emplace_back(VW::cb_class(cost, action, probability));
+                           },
+                       },
+                       *label_value);
+                 }
+
+                 return label;
+               }),
+          py::kw_only(), py::arg("label") = py::none(), py::arg("weight") = 1.f, py::arg("shared") = false, R"docstring(
+    A label representing a contextual bandit problem.
+
+    .. note::
+      Currently the label can only contain 1 or 0 cb costs. There is a mode in vw for CB (non-adf) that allows for multiple cb_classes per example, but it is not currently accessible via direct label access. If creating examples/labels from parsed input it should still work as expected. If you need this feature, please open an issue on the github repo.
+
+    Args:
+      label (Optional[Union[Tuple[float, float], Tuple[int, float, float]]): This is (action, cost, probability). The same rules as VW apply for if the action can be left out of the tuple.
+      weight (float): The weight of the example.
+      shared (bool): Whether the example is shared. This is only used for ADF examples and must be the first example. There can only be one shared example per ADF example list.
+)docstring")
+      .def_property_readonly(
+          "shared", [](VW::cb_label& l) -> bool { return l.costs.size() == 1 && l.costs[0].probability == -1.f; },
+          R"docstring(
+    Whether the example is shared. This is only used for ADF examples and must be the first example. There can only be one shared example per ADF example list.
+)docstring")
+      .def_property(
+          "label",
+          [](VW::cb_label& l) -> std::optional<std::tuple<uint32_t, float, float>>
+          {
+            if (l.costs.size() == 0) { return std::nullopt; }
+            if (l.costs.size() == 1 && l.costs[0].probability == -1.f) { return std::nullopt; }
+            return std::make_tuple(l.costs[0].action, l.costs[0].cost, l.costs[0].probability);
+          },
+          [](VW::cb_label& l,
+              std::optional<std::variant<std::tuple<float, float>, std::tuple<uint32_t, float, float>>> label_value)
+          {
+            if (l.costs.size() == 1 && l.costs[0].probability == -1.f)
+            {
+              throw std::invalid_argument("Shared examples cannot have action, cost, or probability.");
+            }
+            l.costs.clear();
+            if (label_value.has_value())
+            {
+              std::visit(
+                  overloaded{
+                      [&](std::tuple<float, float> value)
+                      {
+                        auto [cost, probability] = value;
+                        l.costs.emplace_back(VW::cb_class(cost, 0, probability));
+                      },
+                      [&](std::tuple<uint32_t, float, float> value)
+                      {
+                        auto [action, cost, probability] = value;
+                        l.costs.emplace_back(VW::cb_class(cost, action, probability));
+                      },
+                  },
+                  *label_value);
+            }
+          },
+          R"docstring(
+    The label for the example. The format of the label is (action, cost, probability). If the action is not specified, it will be set to 0.
+)docstring")
+      .def_readwrite("weight", &VW::cb_label::weight, R"docstring(
+    The weight of the example.
+)docstring");
+
+  py::class_<VW::cs_label>(m, "CSLabel")
+      .def(py::init(
+               [](std::optional<std::vector<std::tuple<float, float>>> costs, bool is_shared)
+               {
+                 auto label = std::make_unique<VW::cs_label>();
+                 if (is_shared)
+                 {
+                   if (costs.has_value())
+                   {
+                     throw std::invalid_argument("Shared examples cannot have action, cost, or probability.");
+                   }
+
+                   label->costs.emplace_back(-FLT_MAX, 0, 0.f, 0.f);
+                   return label;
+                 }
+
+                 if (costs.has_value())
+                 {
+                   for (auto& [class_index, cost] : *costs) { label->costs.emplace_back(cost, class_index, 0.f, 0.f); }
+                 }
+
+                 return label;
+               }),
+          py::kw_only(), py::arg("costs") = py::none(), py::arg("shared") = false, R"docstring(
+    A label representing a cost sensitive classification problem.
+
+    Args:
+      costs (Optional[List[Tuple[int, float]]]): List of classes and costs. If there is no label, this should be None.
+      shared (bool): Whether the example represents the shared context
+)docstring")
+      .def_property_readonly(
+          "shared", [](VW::cs_label& l) -> bool { return is_shared(l); },
+          R"docstring(
+    Whether the example represents the shared context.
+)docstring")
+      .def_property(
+          "costs",
+          [](VW::cs_label& l) -> std::optional<std::vector<std::tuple<uint32_t, float>>>
+          {
+            if (is_shared(l)) { return std::nullopt; }
+
+            std::vector<std::tuple<uint32_t, float>> costs;
+            costs.reserve(l.costs.size());
+            for (auto& cost : l.costs) { costs.emplace_back(cost.class_index, cost.x); }
+            return costs;
+          },
+          [](VW::cs_label& l, const std::vector<std::tuple<uint32_t, float>>& label)
+          {
+            if (is_shared(l)) { throw std::invalid_argument("Shared examples cannot have costs."); }
+
+            l.costs.clear();
+            for (auto& [class_index, cost] : label) { l.costs.emplace_back(cost, class_index, 0.f, 0.f); }
+          },
+          R"docstring(
+    The costs for the example. The format of the costs is (class_index, cost).
+)docstring");
+
   py::class_<VW::example, std::shared_ptr<VW::example>>(m, "Example")
       .def(py::init(
           []()
@@ -642,13 +1006,74 @@ PYBIND11_MODULE(_core, m)
             // shared ptr which returns to the pool upon deletion
             return get_example_from_pool();
           }))
-      .def("_is_newline", [](VW::example& ex) -> bool { return ex.is_newline; });
+      .def("_is_newline", [](VW::example& ex) -> bool { return ex.is_newline; })
+      .def("_get_label",
+          [](VW::example& ex, VW::label_type_t label_type)
+              -> std::variant<py_simple_label, VW::multiclass_label, VW::cb_label, VW::cs_label, std::monostate>
+          {
+            switch (label_type)
+            {
+              case VW::label_type_t::SIMPLE:
+              {
+                const auto& simple_red_features = ex.ex_reduction_features.get<VW::simple_label_reduction_features>();
+                return py_simple_label{ex.l.simple.label, simple_red_features.weight, simple_red_features.initial};
+              }
+              case VW::label_type_t::MULTICLASS:
+                return ex.l.multi;
+              case VW::label_type_t::CB:
+                return ex.l.cb;
+              case VW::label_type_t::CS:
+                return ex.l.cs;
+              case VW::label_type_t::NOLABEL:
+                return std::monostate();
+              default:
+                throw std::runtime_error("Unsupported label type");
+            }
+          })
+      .def("_set_label",
+          [](VW::example& ex,
+              const std::variant<py_simple_label*, VW::multiclass_label*, VW::cb_label*, VW::cs_label*, std::monostate>&
+                  label) -> void
+          {
+            std::visit(
+                overloaded{
+                    // Corresponds to nolabel
+                    [](std::monostate) {},
+                    [&](py_simple_label* simple_label)
+                    {
+                      ex.l.simple.label = simple_label->label;
+                      ex.ex_reduction_features.get<VW::simple_label_reduction_features>().weight = simple_label->weight;
+                      ex.ex_reduction_features.get<VW::simple_label_reduction_features>().initial =
+                          simple_label->initial;
+                    },
+                    [&](VW::multiclass_label* multiclass_label) { ex.l.multi = *multiclass_label; },
+                    [&](VW::cb_label* cb_label) { ex.l.cb = *cb_label; },
+                    [&](VW::cs_label* cs_label) { ex.l.cs = *cs_label; },
+                },
+                label);
+          })
+      .def("_get_tag", [](VW::example& ex) -> std::string { return std::string(ex.tag.data(), ex.tag.size()); });
 
   py::class_<workspace_with_logger_contexts>(m, "Workspace")
       .def(py::init(
-               [](const std::vector<std::string>& args, const std::optional<py::bytes>& bytes)
+               [](const std::vector<std::string>& args, const std::optional<py::bytes>& bytes,
+                   bool record_feature_names, bool record_metrics)
                {
                  auto opts = std::make_unique<VW::config::options_cli>(args);
+                 if (record_metrics)
+                 {
+                   // VW enables metrics by passing a file to write the metrics to.
+                   opts->insert("extra_metrics", "THIS_FILE_SHOULD_NOT_EXIST1");
+                 }
+
+                 if (record_feature_names)
+                 {
+                   // We need to ensure hash_inv is enabled during initialize for things to work correctly.
+                   // The only way to do that is via an option. We're using this one
+                   opts->insert("dump_json_weights_experimental", "THIS_FILE_SHOULD_NOT_EXIST2");
+                   opts->insert("dump_json_weights_include_feature_names_experimental", "");
+                 }
+
                  std::unique_ptr<VW::io::reader> model_reader = nullptr;
                  std::string_view bytes_view;
                  if (bytes.has_value())
@@ -695,7 +1120,8 @@ PYBIND11_MODULE(_core, m)
 
                  return wrapped_object;
                }),
-          py::arg("args"), py::kw_only(), py::arg("model_data") = std::nullopt)
+          py::arg("args"), py::kw_only(), py::arg("model_data") = std::nullopt, py::arg("record_feature_names") = false,
+          py::arg("record_metrics") = false)
       .def(
           "learn_one",
           [](workspace_with_logger_contexts& workspace, VW::example& example) -> void
@@ -706,12 +1132,12 @@ PYBIND11_MODULE(_core, m)
             // Learner is used directly as VW makes decisions about training and
             // learn returns prediction in the workspace API and ends up calling
             // potentially the wrong thing.
-            auto* learner = VW::LEARNER::as_singleline(workspace.workspace_ptr->l);
+            auto* learner = VW::LEARNER::require_singleline(workspace.workspace_ptr->l.get());
+            assert(learner != nullptr);
             learner->learn(example);
 
             // TODO - when updating VW submodule if learn calls update stats then remove this to avoid a double call.
-            update_stats_recursive(
-                *workspace.workspace_ptr, *VW::LEARNER::as_singleline(workspace.workspace_ptr->l), example);
+            update_stats_recursive(*workspace.workspace_ptr, *learner, example);
           },
           py::arg("examples"), py::kw_only())
       .def(
@@ -725,12 +1151,12 @@ PYBIND11_MODULE(_core, m)
             // Learner is used directly as VW makes decisions about training and
             // learn returns prediction in the workspace API and ends up calling
             // potentially the wrong thing.
-            auto* learner = VW::LEARNER::as_multiline(workspace.workspace_ptr->l);
+            auto* learner = VW::LEARNER::require_multiline(workspace.workspace_ptr->l.get());
+            assert(learner != nullptr);
             learner->learn(example);
 
             // TODO - when updating VW submodule if learn calls update stats then remove this to avoid a double call.
-            update_stats_recursive(
-                *workspace.workspace_ptr, *VW::LEARNER::as_multiline(workspace.workspace_ptr->l), example);
+            update_stats_recursive(*workspace.workspace_ptr, *learner, example);
           },
           py::arg("examples"), py::kw_only())
       .def(
@@ -745,12 +1171,11 @@ PYBIND11_MODULE(_core, m)
             // Learner is used directly as VW makes decisions about training and
             // learn returns prediction in the workspace API and ends up calling
             // potentially the wrong thing.
-            auto* learner = VW::LEARNER::as_singleline(workspace.workspace_ptr->l);
+            auto* learner = VW::LEARNER::require_singleline(workspace.workspace_ptr->l.get());
             learner->predict(example);
 
             // TODO - when updating VW submodule if learn calls update stats then remove this to avoid a double call.
-            update_stats_recursive(
-                *workspace.workspace_ptr, *VW::LEARNER::as_singleline(workspace.workspace_ptr->l), example);
+            update_stats_recursive(*workspace.workspace_ptr, *learner, example);
             example.test_only = test_only;
             return to_prediction(example.pred, workspace.workspace_ptr->l->get_output_prediction_type());
           },
@@ -770,12 +1195,11 @@ PYBIND11_MODULE(_core, m)
             // Learner is used directly as VW makes decisions about training and
             // learn returns prediction in the workspace API and ends up calling
             // potentially the wrong thing.
-            auto* learner = VW::LEARNER::as_multiline(workspace.workspace_ptr->l);
+            auto* learner = VW::LEARNER::require_multiline(workspace.workspace_ptr->l.get());
             learner->predict(example);
 
             // TODO - when updating VW submodule if learn calls update stats then remove this to avoid a double call.
-            update_stats_recursive(
-                *workspace.workspace_ptr, *VW::LEARNER::as_multiline(workspace.workspace_ptr->l), example);
+            update_stats_recursive(*workspace.workspace_ptr, *learner, example);
             for (size_t i = 0; i < example.size(); i++) { example[i]->test_only = test_onlys[i]; }
             return to_prediction(example[0]->pred, workspace.workspace_ptr->l->get_output_prediction_type());
           },
@@ -787,12 +1211,12 @@ PYBIND11_MODULE(_core, m)
             py_setup_example(*workspace.workspace_ptr, example);
             auto on_exit = VW::scope_exit([&]() { py_unsetup_example(*workspace.workspace_ptr, example); });
 
+            auto* learner = VW::LEARNER::require_singleline(workspace.workspace_ptr->l.get());
             if (workspace.workspace_ptr->l->learn_returns_prediction)
             {
               // Learner is used directly as VW makes decisions about training and
               // learn returns prediction in the workspace API and ends up calling
               // potentially the wrong thing.
-              auto* learner = VW::LEARNER::as_singleline(workspace.workspace_ptr->l);
               learner->learn(example);
             }
             else
@@ -800,7 +1224,6 @@ PYBIND11_MODULE(_core, m)
               // Learner is used directly as VW makes decisions about training and
               // learn returns prediction in the workspace API and ends up calling
               // potentially the wrong thing.
-              auto* learner = VW::LEARNER::as_singleline(workspace.workspace_ptr->l);
               // We must save and restore test_only because the library sets this values and does not undo it.
               bool test_only = example.test_only;
               learner->predict(example);
@@ -810,8 +1233,7 @@ PYBIND11_MODULE(_core, m)
             }
 
             // TODO - when updating VW submodule if learn calls update stats then remove this to avoid a double call.
-            update_stats_recursive(
-                *workspace.workspace_ptr, *VW::LEARNER::as_singleline(workspace.workspace_ptr->l), example);
+            update_stats_recursive(*workspace.workspace_ptr, *learner, example);
             return to_prediction(example.pred, workspace.workspace_ptr->l->get_output_prediction_type());
           },
           py::arg("examples"), py::kw_only())
@@ -821,13 +1243,12 @@ PYBIND11_MODULE(_core, m)
           {
             py_setup_example(*workspace.workspace_ptr, example);
             auto on_exit = VW::scope_exit([&]() { py_unsetup_example(*workspace.workspace_ptr, example); });
-
+            auto* learner = VW::LEARNER::require_multiline(workspace.workspace_ptr->l.get());
             if (workspace.workspace_ptr->l->learn_returns_prediction)
             {
               // Learner is used directly as VW makes decisions about training and
               // learn returns prediction in the workspace API and ends up calling
               // potentially the wrong thing.
-              auto* learner = VW::LEARNER::as_multiline(workspace.workspace_ptr->l);
               learner->learn(example);
             }
             else
@@ -835,7 +1256,6 @@ PYBIND11_MODULE(_core, m)
               // Learner is used directly as VW makes decisions about training and
               // learn returns prediction in the workspace API and ends up calling
               // potentially the wrong thing.
-              auto* learner = VW::LEARNER::as_multiline(workspace.workspace_ptr->l);
               // We must save and restore test_only because the library sets this values and does not undo it.
               std::vector<bool> test_onlys;
               test_onlys.reserve(example.size());
@@ -847,13 +1267,25 @@ PYBIND11_MODULE(_core, m)
             }
 
             // TODO - when updating VW submodule if learn calls update stats then remove this to avoid a double call.
-            update_stats_recursive(
-                *workspace.workspace_ptr, *VW::LEARNER::as_multiline(workspace.workspace_ptr->l), example);
+            update_stats_recursive(*workspace.workspace_ptr, *learner, example);
             return to_prediction(example[0]->pred, workspace.workspace_ptr->l->get_output_prediction_type());
           },
           py::arg("examples"), py::kw_only())
       .def("get_is_multiline",
           [](const workspace_with_logger_contexts& workspace) { return workspace.workspace_ptr->l->is_multiline(); })
+      .def("get_metrics",
+          [](const workspace_with_logger_contexts& workspace) -> py::dict
+          {
+            if (!workspace.workspace_ptr->global_metrics.are_metrics_enabled())
+            {
+              throw std::runtime_error(
+                  "Metrics are not enabled. Pass records_metrics=True to Workspace constructor to enable.");
+            }
+            py::dict metrics;
+            auto collected_metrics =
+                workspace.workspace_ptr->global_metrics.collect_metrics(workspace.workspace_ptr->l.get());
+            return convert_metrics_to_dict(collected_metrics);
+          })
       .def("get_prediction_type",
           [](const workspace_with_logger_contexts& workspace)
           { return workspace.workspace_ptr->l->get_output_prediction_type(); })
@@ -929,10 +1361,35 @@ PYBIND11_MODULE(_core, m)
                 });
             return workspace.workspace_ptr->dump_weights_to_json_experimental();
           },
-          py::kw_only(), py::arg("include_feature_names") = false, py::arg("include_online_state") = false);
+          py::kw_only(), py::arg("include_feature_names") = false, py::arg("include_online_state") = false)
+      .def(
+          "readable_model",
+          [](const workspace_with_logger_contexts& workspace, bool include_feature_names) -> std::string
+          {
+            auto& all = *workspace.workspace_ptr;
+            if (include_feature_names)
+            {
+              if (!all.hash_inv)
+              {
+                THROW(
+                    "record_feature_names must be enabled (from Workspace constructor) to use "
+                    "include_feature_names=True");
+              }
+            }
+
+            auto print_invert_guard = VW::swap_guard(all.print_invert, include_feature_names);
+            VW::io_buf buffer;
+            auto vec_buffer = std::make_shared<std::vector<char>>();
+            buffer.add_file(VW::io::create_vector_writer(vec_buffer));
+            VW::details::dump_regressor(all, buffer, true);
+            buffer.flush();
+            return std::string(vec_buffer->data(), vec_buffer->size());
+          },
+          py::kw_only(), py::arg("include_feature_names") = false);
 
   m.def("_parse_line_text", &::parse_text_line, py::arg("workspace"), py::arg("line"));
   m.def("_parse_line_dsjson", &::parse_dsjson_line, py::arg("workspace"), py::arg("line"));
+  m.def("_parse_line_json", &::parse_json_line, py::arg("workspace"), py::arg("line"));
   m.def("_write_cache_header", &::write_cache_header, py::arg("workspace"), py::arg("file"));
   m.def("_write_cache_example", &::write_cache_example, py::arg("workspace"), py::arg("example"), py::arg("file"));
   m.def("_run_cli_driver", &::run_cli_driver, py::arg("args"), py::kw_only(), py::arg("onethread") = false);
